/* eslint-disable react-hooks/exhaustive-deps */
/**
 * SwaggerApi
 * Swagger API, fetching data
 * and providing apiContext and userContext
 */
import React, { useCallback, useRef } from "react";
import SwaggerClient from "swagger-client";
import { createContext, useContext, useEffect, useState } from "react";
import { CircularProgress } from "@mui/material";
import { useCustomSnackbar } from "../components/common/snackbar/Snackbar";
import { userContext } from "@ess-ics/ce-ui-common";

const createSwaggerClient = async () => {
  const swaggerOptions = {
    url: `${window.SERVER_ADDRESS}${window.API_BASE_ENDPOINT}`,
    requestInterceptor: (req) => {
      // if (user) req.headers["Authorization"] = `Bearer ${user.token}`;
      req.headers["Content-Type"] = "application/json";
      req.headers.accept = "application/json";
      window.req = req;
      return req;
    },
    responseInterceptor: (res) => {
      return res;
    }
  };

  const swagger = await SwaggerClient(swaggerOptions);
  // update the server url to be our intended server
  swagger.spec.servers[0].url = `${window.SERVER_ADDRESS}`;
  window.swagger = swagger;
  return swagger;
};

export function useAPI() {
  const create = useCallback(createSwaggerClient, []);
  return useAsync({ fcn: create });
}

export const apiContext = createContext(null);

export function LegacyApiProvider({ children }) {
  const [api] = useAPI();

  return api ? (
    <apiContext.Provider value={api}>{children}</apiContext.Provider>
  ) : (
    <CircularProgress />
  );
}

export function useAPIErrorHandler(onError) {
  const { logout } = useContext(userContext);
  const showError = useCustomSnackbar();

  function errorHandler(e) {
    const { response, message } = e;
    const { obj, status } = response ?? { obj: {}, status: "" };

    // Always log out if 401
    if (status === 401) {
      logout();
    }

    if (onError) {
      const errorNotHandled = onError(obj?.description ?? message, status);
      if (errorNotHandled) {
        showError(obj?.description ?? message);
      }
    } else {
      showError(obj?.description ?? message);
    }
  }

  return useCallback(errorHandler, [logout]);
}

export function useQueuedResponse({
  init = null,
  onError = null,
  initLoading = false
}) {
  const queueRef = useRef([]);
  const busyRef = useRef(false);
  const [response, setResponse] = useState(init);
  const [loading, setLoading] = useState(initLoading);
  const errorHandler = useAPIErrorHandler(onError);
  const MAX_QUEUE_LENGTH = 10;

  const processQueue = useCallback(async () => {
    if (busyRef.current) {
      return;
    }

    busyRef.current = true;
    setLoading(true);

    let responses = [];
    const queue = queueRef.current;
    while (responses.length !== queue.length) {
      responses = await Promise.allSettled(queue);
    }

    queueRef.current = [];
    busyRef.current = false;

    const { status, value, reason } = responses.pop();
    if (status === "rejected") {
      setLoading(false);
      errorHandler(reason);
      return;
    }

    setResponse(value);
    setLoading(false);
  }, [errorHandler]);

  const enqueue = useCallback(
    (promise) => {
      if (queueRef.current.length > MAX_QUEUE_LENGTH) {
        throw new Error(
          `Max queue length (${MAX_QUEUE_LENGTH}) exceeded in useQueuedResponse.`
        );
      }
      queueRef.current.push(promise);
      processQueue();
    },
    [queueRef, processQueue]
  );

  return { response, setResponse, loading, enqueue };
}

export function useAsync({ fcn, call = true, init = null, onError = null }) {
  const { response, setResponse, loading, enqueue } = useQueuedResponse({
    init,
    onError,
    call
  });

  const reset = useCallback(() => setResponse(init), [setResponse, init]);

  const wrapper = useCallback(
    (...args) => {
      enqueue(fcn(...args));
    },
    [fcn, enqueue]
  );

  useEffect(() => {
    if (call) wrapper();
  }, [wrapper, call]);

  return [response, wrapper, reset, loading];
}

function useCallAndUnpack(fcn, unpacker = (x) => x) {
  return useCallback(async (...args) => {
    const response = await fcn(...args);
    const unpacked = unpacker(response.obj, response.status);
    return unpacked;
  }, []);
}

export function unpackIOC(ioc) {
  ioc = { ...ioc };
  const {
    id,
    description,
    createdBy,
    active,
    activeDeployment,
    namingName,
    gitProjectId,
    namingUuid,
    sourceUrl,
    sourceVersion,
    sourceVersionShort,
    operationInProgress,
    alerts
  } = ioc;

  let unpackedIOC = {
    id: id,
    description: description,
    createdBy: createdBy,
    activeDeployment: activeDeployment,
    active: active,
    gitProjectId: gitProjectId,
    sourceUrl: sourceUrl,
    sourceVersion: sourceVersion,
    sourceVersionShort: sourceVersionShort,
    namingName: namingName,
    operationInProgress: operationInProgress,
    alerts: alerts,
    namingUuid: namingUuid
  };

  return unpackedIOC;
}

export function unpackIocInfo(ioc) {
  ioc = { ...ioc };
  return ioc;
}

export function unpackIOCList(iocs) {
  let iocArray = iocs.iocList.map((ioc) => unpackIocInfo(ioc));

  let unpackedIOCList = {
    totalCount: iocs.totalCount,
    pageNumber: iocs.pageNumber,
    limit: iocs.limit,
    iocList: iocArray
  };

  return unpackedIOCList;
}

export function unpackDeployment(deployment) {
  const d = { ...deployment };
  return d;
}

export function unpackOngoingOperations(input) {
  const { operations: operationsList, ...rest } = input;
  const output = { ...rest, operationsList };
  return output.totalCount > 0 ? output.operationsList[0] : null;
}

export function unpackJob(job) {
  return { ...job };
}

export function useJobLogById() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack((awxJobId) =>
    api.apis.Deployments.fetchDeploymentJobLog({ awx_job_id: awxJobId })
  );
  return useAsync({ fcn: method, call: false, init: null });
}

export function unpackRecord(record) {
  return { ...record };
}

export function useStartIOC(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.IOCs.startIoc);
  const boundMethod = useCallback(method.bind(null, { ioc_id: id }), [id]);
  return useAsync({ fcn: boundMethod, call: false, onError: onError });
}

export function useStopIOC(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.IOCs.stopIoc);
  const boundMethod = useCallback(method.bind(null, { ioc_id: id }), [id]);
  return useAsync({ fcn: boundMethod, call: false, onError: onError });
}

export function useRenewToken() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(api.apis.Authentication.tokenRenew);

  const [, /* response*/ renewToken /* reset*/, , loading] = useAsync({
    fcn: method,
    call: false
  });
  return [renewToken, loading];
}

export function unpackGitReference(reference) {
  return { ...reference };
}

export function unpackTagAndCommitIdList(tagList) {
  return tagList.map((t) => unpackGitReference(t));
}

export function useTagsAndCommitIds(onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (gitProjectId, reference, includeAllReference, searchMethod) =>
      api.apis.Git.listTagsAndCommitIds({
        project_id: gitProjectId,
        reference: reference,
        include_all_reference: includeAllReference,
        search_method: searchMethod
      }),
    unpackTagAndCommitIdList
  );
  return useAsync({ fcn: method, call: false, onError: onError, init: [] });
}

export function unpackGitProject(project) {
  return { ...project };
}

export function unpackAllowedGitProjects(projectList) {
  return projectList.map((p) => unpackGitProject(p));
}

export function useAllowedGitProjects() {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (repoUrl) => api.apis.Git.listProjects(),
    unpackAllowedGitProjects
  );
  return useAsync({ fcn: method, call: false, init: [] });
}

export function useDeleteIOC(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    api.apis.IOCs.deleteIoc,
    unpackDeleteIocResponse
  );
  const boundMethod = useCallback(method.bind(null, { ioc_id: id }), [id]);
  return useAsync({ fcn: boundMethod, call: false, onError: onError });
}

export function unpackDeleteIocResponse(_, status) {
  if (status === 204) {
    return "Successful delete";
  }
}

export function useUpdateIoc(id, onError) {
  const api = useContext(apiContext);
  const method = useCallAndUnpack(
    (body) => api.apis.IOCs.updateIoc({ ioc_id: id }, { requestBody: body }),
    unpackUpdateIoc
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackUpdateIoc(updateIoc) {
  const d = { ...updateIoc };
  return d;
}

export function unpackUpdateActiveDeploymentHost(ioc) {
  return { ...ioc };
}
